@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DiStella.cxx 2263 2011-07-13 14:13:49Z stephena $
+// $Id: DiStella.cxx 2417 2012-03-16 13:19:29Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -818,7 +818,7 @@ int DiStella::mark(uInt32 address, uInt8 mask, bool directive)
     if(directive)  myDirectives[address-myOffset] = mask;
     return 1;
   }
-  else if (address >= 0 && address <= 0x3f)
+  else if (address <= 0x3f)
   {
     return 2;
   }
@@ -976,7 +976,7 @@ void DiStella::processDirectives(const CartDebug::DirectiveList& directives)
   for(CartDebug::DirectiveList::const_iterator i = directives.begin();
       i != directives.end(); ++i)
   {
-    const CartDebug::DirectiveTag tag = *i;
+    const CartDebug::DirectiveTag& tag = *i;
     if(check_range(tag.start, tag.end))
       for(uInt32 k = tag.start; k <= tag.end; ++k)
         mark(k, tag.type, true);
