@@ -8,19 +8,20 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RiotDebug.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: RiotDebug.cxx 2343 2012-01-08 16:55:10Z stephena $
 //============================================================================
 
 #include <sstream>
 
 #include "System.hxx"
 #include "M6532.hxx"
+#include "TIA.hxx"
 #include "Debugger.hxx"
 #include "Switches.hxx"
 
@@ -39,11 +40,23 @@ const DebuggerState& RiotDebug::getState()
   myState.SWCHA_R = swcha();
   myState.SWCHA_W = mySystem.m6532().myOutA;
   myState.SWACNT  = swacnt();
-  myState.SWCHB   = swchb();
+  myState.SWCHB_R = swchb();
+  myState.SWCHB_W = mySystem.m6532().myOutB;
+  myState.SWBCNT  = swbcnt();
   Debugger::set_bits(myState.SWCHA_R, myState.swchaReadBits);
   Debugger::set_bits(myState.SWCHA_W, myState.swchaWriteBits);
   Debugger::set_bits(myState.SWACNT, myState.swacntBits);
-  Debugger::set_bits(myState.SWCHB, myState.swchbBits);
+  Debugger::set_bits(myState.SWCHB_R, myState.swchbReadBits);
+  Debugger::set_bits(myState.SWCHB_W, myState.swchbWriteBits);
+  Debugger::set_bits(myState.SWBCNT, myState.swbcntBits);
+
+  // TIA INPTx registers
+  myState.INPT0 = inpt(0);
+  myState.INPT1 = inpt(1);
+  myState.INPT2 = inpt(2);
+  myState.INPT3 = inpt(3);
+  myState.INPT4 = inpt(4);
+  myState.INPT5 = inpt(5);
 
   // Timer registers
   myState.TIM1T    = tim1T();
@@ -54,20 +67,6 @@ const DebuggerState& RiotDebug::getState()
   myState.TIMINT   = timint();
   myState.TIMCLKS  = timClocks();
 
-  // Controller port pins
-  const Controller& port0 = myConsole.controller(Controller::Left);
-  myState.P0_PIN1 = port0.myDigitalPinState[Controller::One];
-  myState.P0_PIN2 = port0.myDigitalPinState[Controller::Two];
-  myState.P0_PIN3 = port0.myDigitalPinState[Controller::Three];
-  myState.P0_PIN4 = port0.myDigitalPinState[Controller::Four];
-  myState.P0_PIN6 = port0.myDigitalPinState[Controller::Six];
-  const Controller& port1 = myConsole.controller(Controller::Right);
-  myState.P1_PIN1 = port1.myDigitalPinState[Controller::One];
-  myState.P1_PIN2 = port1.myDigitalPinState[Controller::Two];
-  myState.P1_PIN3 = port1.myDigitalPinState[Controller::Three];
-  myState.P1_PIN4 = port1.myDigitalPinState[Controller::Four];
-  myState.P1_PIN6 = port1.myDigitalPinState[Controller::Six];
-
   return myState;
 }
 
@@ -78,11 +77,23 @@ void RiotDebug::saveOldState()
   myOldState.SWCHA_R = swcha();
   myOldState.SWCHA_W = mySystem.m6532().myOutA;
   myOldState.SWACNT  = swacnt();
-  myOldState.SWCHB   = swchb();
+  myOldState.SWCHB_R = swchb();
+  myOldState.SWCHB_W = mySystem.m6532().myOutB;
+  myOldState.SWBCNT  = swbcnt();
   Debugger::set_bits(myOldState.SWCHA_R, myOldState.swchaReadBits);
   Debugger::set_bits(myOldState.SWCHA_W, myOldState.swchaWriteBits);
   Debugger::set_bits(myOldState.SWACNT, myOldState.swacntBits);
-  Debugger::set_bits(myOldState.SWCHB, myOldState.swchbBits);
+  Debugger::set_bits(myOldState.SWCHB_R, myOldState.swchbReadBits);
+  Debugger::set_bits(myOldState.SWCHB_W, myOldState.swchbWriteBits);
+  Debugger::set_bits(myOldState.SWBCNT, myOldState.swbcntBits);
+
+  // TIA INPTx registers
+  myOldState.INPT0 = inpt(0);
+  myOldState.INPT1 = inpt(1);
+  myOldState.INPT2 = inpt(2);
+  myOldState.INPT3 = inpt(3);
+  myOldState.INPT4 = inpt(4);
+  myOldState.INPT5 = inpt(5);
 
   // Timer registers
   myOldState.TIM1T    = tim1T();
@@ -92,20 +103,6 @@ void RiotDebug::saveOldState()
   myOldState.INTIM    = intim();
   myOldState.TIMINT   = timint();
   myOldState.TIMCLKS  = timClocks();
-
-  // Controller port pins
-  const Controller& port0 = myConsole.controller(Controller::Left);
-  myOldState.P0_PIN1 = port0.myDigitalPinState[Controller::One];
-  myOldState.P0_PIN2 = port0.myDigitalPinState[Controller::Two];
-  myOldState.P0_PIN3 = port0.myDigitalPinState[Controller::Three];
-  myOldState.P0_PIN4 = port0.myDigitalPinState[Controller::Four];
-  myOldState.P0_PIN6 = port0.myDigitalPinState[Controller::Six];
-  const Controller& port1 = myConsole.controller(Controller::Right);
-  myOldState.P1_PIN1 = port1.myDigitalPinState[Controller::One];
-  myOldState.P1_PIN2 = port1.myDigitalPinState[Controller::Two];
-  myOldState.P1_PIN3 = port1.myDigitalPinState[Controller::Three];
-  myOldState.P1_PIN4 = port1.myDigitalPinState[Controller::Four];
-  myOldState.P1_PIN6 = port1.myDigitalPinState[Controller::Six];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -120,6 +117,9 @@ uInt8 RiotDebug::swcha(int newVal)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 RiotDebug::swchb(int newVal)
 {
+  if(newVal > -1)
+    mySystem.poke(0x282, newVal);
+
   return mySystem.peek(0x282);
 }
 
@@ -132,6 +132,34 @@ uInt8 RiotDebug::swacnt(int newVal)
   return mySystem.peek(0x281);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 RiotDebug::swbcnt(int newVal)
+{
+  if(newVal > -1)
+    mySystem.poke(0x283, newVal);
+
+  return mySystem.peek(0x283);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 RiotDebug::inpt(int x)
+{
+  static TIARegister _inpt[6] = { INPT0, INPT1, INPT2, INPT3, INPT4, INPT5 };
+  return mySystem.peek(_inpt[x]);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool RiotDebug::vblank(int bit)
+{
+  if(bit == 6)       // latches
+    return mySystem.tia().myVBLANK & 0x40;
+  else if(bit == 7)  // dump to ground
+    return mySystem.tia().myDumpEnabled;
+  else
+    return true;
+}
+
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 RiotDebug::tim1T(int newVal)
 {
@@ -187,25 +215,9 @@ Int32 RiotDebug::timClocks()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RiotDebug::setP0Pins(bool Pin1, bool Pin2, bool Pin3, bool Pin4, bool Pin6)
+Controller& RiotDebug::controller(Controller::Jack jack) const
 {
-  Controller& port0 = myConsole.controller(Controller::Left);
-  port0.myDigitalPinState[Controller::One]   = Pin1;
-  port0.myDigitalPinState[Controller::Two]   = Pin2;
-  port0.myDigitalPinState[Controller::Three] = Pin3;
-  port0.myDigitalPinState[Controller::Four]  = Pin4;
-  port0.myDigitalPinState[Controller::Six]   = Pin6;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RiotDebug::setP1Pins(bool Pin1, bool Pin2, bool Pin3, bool Pin4, bool Pin6)
-{
-  Controller& port1 = myConsole.controller(Controller::Right);
-  port1.myDigitalPinState[Controller::One]   = Pin1;
-  port1.myDigitalPinState[Controller::Two]   = Pin2;
-  port1.myDigitalPinState[Controller::Three] = Pin3;
-  port1.myDigitalPinState[Controller::Four]  = Pin4;
-  port1.myDigitalPinState[Controller::Six]   = Pin6;
+  return myConsole.controller(jack);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -326,8 +338,12 @@ string RiotDebug::toString()
       << myDebugger.invIfChanged(state.SWCHA_W, oldstate.SWCHA_W) << " "
       << myDebugger.valueToString(0x281) + "/SWACNT="
       << myDebugger.invIfChanged(state.SWACNT, oldstate.SWACNT) << " "
-      << myDebugger.valueToString(0x282) + "/SWCHB="
-      << myDebugger.invIfChanged(state.SWCHB, oldstate.SWCHB) << " "
+      << myDebugger.valueToString(0x282) + "/SWCHB(R)="
+      << myDebugger.invIfChanged(state.SWCHB_R, oldstate.SWCHB_R) << " "
+      << myDebugger.valueToString(0x282) + "/SWCHB(W)="
+      << myDebugger.invIfChanged(state.SWCHB_W, oldstate.SWCHB_W) << " "
+      << myDebugger.valueToString(0x283) + "/SWBCNT="
+      << myDebugger.invIfChanged(state.SWBCNT, oldstate.SWBCNT) << " "
       << endl
 
       // These are squirrely: some symbol files will define these as
