@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaInfoWidget.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: TiaInfoWidget.cxx 2343 2012-01-08 16:55:10Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -111,7 +111,7 @@ TiaInfoWidget::~TiaInfoWidget()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TiaInfoWidget::handleMouseDown(int x, int y, int button, int clickCount)
 {
-cerr << "TiaInfoWidget button press: x = " << x << ", y = " << y << endl;
+//cerr << "TiaInfoWidget button press: x = " << x << ", y = " << y << endl;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
