@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartEF.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: CartEF.cxx 2325 2012-01-02 20:31:42Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -24,10 +24,12 @@
 #include "CartEF.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeEF::CartridgeEF(const uInt8* image)
+CartridgeEF::CartridgeEF(const uInt8* image, uInt32 size, const Settings& settings)
+  : Cartridge(settings)
 {
   // Copy the ROM image into my buffer
-  memcpy(myImage, image, 65536);
+  memcpy(myImage, image, BSPF_min(65536u, size));
+  createCodeAccessBase(65536);
 
   // Remember startup bank
   myStartBank = 1;
@@ -49,21 +51,9 @@ void CartridgeEF::reset()
 void CartridgeEF::install(System& system)
 {
   mySystem = &system;
-  uInt16 shift = mySystem->pageShift();
-  uInt16 mask = mySystem->pageMask();
 
   // Make sure the system we're being installed in has a page size that'll work
-  assert((0x1000 & mask) == 0);
-
-  System::PageAccess access;
-
-  // Set the page accessing methods for the hot spots
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
-  for(uInt32 i = (0x1FE0 & ~mask); i < 0x2000; i += (1 << shift))
-    mySystem->setPageAccess(i >> shift, access);
+  assert((0x1000 & mySystem->pageMask()) == 0);
 
   // Install pages for the startup bank
   bank(myStartBank);
@@ -94,9 +84,9 @@ bool CartridgeEF::poke(uInt16 address, uInt8)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeEF::bank(uInt16 bank)
+bool CartridgeEF::bank(uInt16 bank)
 {
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -104,21 +94,24 @@ void CartridgeEF::bank(uInt16 bank)
   uInt16 shift = mySystem->pageShift();
   uInt16 mask = mySystem->pageMask();
 
-  System::PageAccess access;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
+
+  // Set the page accessing methods for the hot spots
+  for(uInt32 i = (0x1FE0 & ~mask); i < 0x2000; i += (1 << shift))
+  {
+    access.codeAccessBase = &myCodeAccessBase[offset + (i & 0x0FFF)];
+    mySystem->setPageAccess(i >> shift, access);
+  }
 
   // Setup the page access methods for the current bank
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
-
-  // Map ROM image into the system
   for(uInt32 address = 0x1000; address < (0x1FE0U & ~mask);
       address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x0FFF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
     mySystem->setPageAccess(address >> shift, access);
   }
-  myBankChanged = true;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -150,12 +143,9 @@ const uInt8* CartridgeEF::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeEF::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
-
+    out.putString(name());
     out.putInt(myCurrentBank);
   }
   catch(const char* msg)
@@ -170,11 +160,9 @@ bool CartridgeEF::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeEF::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     myCurrentBank = (uInt16) in.getInt();
