@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartUA.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: CartUA.cxx 2325 2012-01-02 20:31:42Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -24,10 +24,12 @@
 #include "CartUA.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeUA::CartridgeUA(const uInt8* image)
+CartridgeUA::CartridgeUA(const uInt8* image, uInt32 size, const Settings& settings)
+  : Cartridge(settings)
 {
   // Copy the ROM image into my buffer
-  memcpy(myImage, image, 8192);
+  memcpy(myImage, image, BSPF_min(8192u, size));
+  createCodeAccessBase(8192);
 
   // Remember startup bank
   myStartBank = 0;
@@ -60,11 +62,7 @@ void CartridgeUA::install(System& system)
   myHotSpotPageAccess = mySystem->getPageAccess(0x0220 >> shift);
 
   // Set the page accessing methods for the hot spots
-  System::PageAccess access;
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
   mySystem->setPageAccess(0x0220 >> shift, access);
   mySystem->setPageAccess(0x0240 >> shift, access);
 
@@ -134,9 +132,9 @@ bool CartridgeUA::poke(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeUA::bank(uInt16 bank)
+bool CartridgeUA::bank(uInt16 bank)
 { 
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -144,18 +142,16 @@ void CartridgeUA::bank(uInt16 bank)
   uInt16 shift = mySystem->pageShift();
 
   // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   // Map ROM image into the system
   for(uInt32 address = 0x1000; address < 0x2000; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x0FFF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
     mySystem->setPageAccess(address >> shift, access);
   }
-  myBankChanged = true;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -187,12 +183,9 @@ const uInt8* CartridgeUA::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeUA::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
-
+    out.putString(name());
     out.putInt(myCurrentBank);
   }
   catch(const char* msg)
@@ -207,11 +200,9 @@ bool CartridgeUA::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeUA::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     myCurrentBank = (uInt16)in.getInt();
