@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBuffer.cxx 2312 2011-12-29 19:10:57Z stephena $
+// $Id: FrameBuffer.cxx 2357 2012-01-14 22:14:04Z stephena $
 //============================================================================
 
 #include <algorithm>
@@ -162,10 +162,6 @@ FBInitStatus FrameBuffer::initialize(const string& title,
   else
     return kFailTooLarge;
 
-  // Enable unicode so we can see translated key events
-  // (lowercase vs. uppercase characters)
-  SDL_EnableUNICODE(1);
-
   // Erase any messages from a previous run
   myMsg.counter = 0;
 
@@ -302,7 +298,7 @@ void FrameBuffer::update()
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBuffer::showMessage(const string& message, MessagePosition position,
-                              bool force, uInt32 color)
+                              bool force)
 {
   // Only show messages if they've been enabled
   if(!(force || myOSystem->settings().getBool("uimessages")))
@@ -318,7 +314,7 @@ void FrameBuffer::showMessage(const string& message, MessagePosition position,
   // Precompute the message coordinates
   myMsg.text    = message;
   myMsg.counter = uInt32(myOSystem->frameRate()) << 1; // Show message for 2 seconds
-  myMsg.color   = color;
+  myMsg.color   = kBtnTextColor;
 
   myMsg.w = myOSystem->font().getStringWidth(myMsg.text) + 10;
   myMsg.h = myOSystem->font().getFontHeight() + 8;
@@ -416,8 +412,8 @@ inline void FrameBuffer::drawMessage()
   }
 
   myMsg.surface->setPos(myMsg.x + myImageRect.x(), myMsg.y + myImageRect.y());
-  myMsg.surface->fillRect(0, 0, myMsg.w-2, myMsg.h-4, kBGColor);
-  myMsg.surface->box(0, 0, myMsg.w, myMsg.h-2, kColor, kShadowColor);
+  myMsg.surface->fillRect(1, 1, myMsg.w-2, myMsg.h-2, kBtnColor);
+  myMsg.surface->box(0, 0, myMsg.w, myMsg.h, kColor, kShadowColor);
   myMsg.surface->drawString(&myOSystem->font(), myMsg.text, 4, 4,
                                myMsg.w, myMsg.color, kTextAlignLeft);
   myMsg.counter--;
