@@ -8,27 +8,29 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: M6532.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: M6532.cxx 2412 2012-03-14 01:19:23Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <iostream>
 
 #include "Console.hxx"
+#include "Settings.hxx"
 #include "Switches.hxx"
 #include "System.hxx"
 
 #include "M6532.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-M6532::M6532(const Console& console)
-  : myConsole(console)
+M6532::M6532(const Console& console, const Settings& settings)
+  : myConsole(console),
+    mySettings(settings)
 {
 }
  
@@ -40,9 +42,12 @@ M6532::~M6532()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void M6532::reset()
 {
-  // Randomize the 128 bytes of memory
-  for(uInt32 t = 0; t < 128; ++t)
-    myRAM[t] = mySystem->randGenerator().next();
+  // Initialize the 128 bytes of memory
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 t = 0; t < 128; ++t)
+      myRAM[t] = mySystem->randGenerator().next();
+  else
+    memset(myRAM, 0, 128);
 
   // The timer absolutely cannot be initialized to zero; some games will
   // loop or hang (notably Solaris and H.E.R.O.)
@@ -53,7 +58,7 @@ void M6532::reset()
   myInterruptTriggered = false;
 
   // Zero the I/O registers
-  myDDRA = myDDRB = myOutA = 0x00;
+  myDDRA = myDDRB = myOutA = myOutB = 0x00;
 
   // Zero the timer registers
   myOutTimer[0] = myOutTimer[1] = myOutTimer[2] = myOutTimer[3] = 0x00;
@@ -90,11 +95,7 @@ void M6532::install(System& system, Device& device)
   assert((0x1080 & mask) == 0);
   
   // All accesses are to the given device
-  System::PageAccess access;
-  access.directPeekBase = 0; 
-  access.directPokeBase = 0;
-  access.device = &device;
-  access.type = System::PA_READWRITE;
+  System::PageAccess access(0, 0, 0, &device, System::PA_READWRITE);
 
   // We're installing in a 2600 system
   for(int address = 0; address < 8192; address += (1 << shift))
@@ -116,21 +117,10 @@ uInt8 M6532::peek(uInt16 addr)
 
   switch(addr & 0x07)
   {
-    case 0x00:    // Port A I/O Register (Joystick)
+    case 0x00:    // SWCHA - Port A I/O Register (Joystick)
     {
-      uInt8 value = 0x00;
-
-      Controller& port0 = myConsole.controller(Controller::Left);
-      if(port0.read(Controller::One))   value |= 0x10;
-      if(port0.read(Controller::Two))   value |= 0x20;
-      if(port0.read(Controller::Three)) value |= 0x40;
-      if(port0.read(Controller::Four))  value |= 0x80;
-
-      Controller& port1 = myConsole.controller(Controller::Right);
-      if(port1.read(Controller::One))   value |= 0x01;
-      if(port1.read(Controller::Two))   value |= 0x02;
-      if(port1.read(Controller::Three)) value |= 0x04;
-      if(port1.read(Controller::Four))  value |= 0x08;
+      uInt8 value = (myConsole.controller(Controller::Left).read() << 4) |
+                     myConsole.controller(Controller::Right).read();
 
       // Each pin is high (1) by default and will only go low (0) if either
       //  (a) External device drives the pin low
@@ -139,17 +129,17 @@ uInt8 M6532::peek(uInt16 addr)
       return (myOutA | ~myDDRA) & value;
     }
 
-    case 0x01:    // Port A Data Direction Register 
+    case 0x01:    // SWACNT - Port A Data Direction Register 
     {
       return myDDRA;
     }
 
-    case 0x02:    // Port B I/O Register (Console switches)
+    case 0x02:    // SWCHB - Port B I/O Register (Console switches)
     {
-      return myConsole.switches().read();
+      return (myOutB | ~myDDRB) & (myConsole.switches().read() | myDDRB);
     }
 
-    case 0x03:    // Port B Data Direction Register
+    case 0x03:    // SWBCNT - Port B Data Direction Register
     {
       return myDDRB;
     }
@@ -227,22 +217,31 @@ bool M6532::poke(uInt16 addr, uInt8 value)
   {
     switch(addr & 0x03)
     {
-      case 0:     // Port A I/O Register (Joystick)
+      case 0:     // SWCHA - Port A I/O Register (Joystick)
       {
         myOutA = value;
-        setPinState();
+        setPinState(true);
         break;
       }
 
-      case 1:     // Port A Data Direction Register 
+      case 1:     // SWACNT - Port A Data Direction Register 
       {
         myDDRA = value;
-        setPinState();
+        setPinState(false);
         break;
       }
 
-      default:    // Port B I/O & DDR Registers (Console switches)
-        return false;  // hardwired as read-only
+      case 2:     // SWCHB - Port B I/O Register (Console switches)
+      {
+        myOutB = value;
+        break;
+      }
+
+      case 3:     // SWBCNT - Port B Data Direction Register 
+      {
+        myDDRB = value;
+        break;
+      }
     }
   }
   return true;
@@ -261,7 +260,7 @@ void M6532::setTimerRegister(uInt8 value, uInt8 interval)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void M6532::setPinState()
+void M6532::setPinState(bool swcha)
 {
   /*
     When a bit in the DDR is set as input, +5V is placed on its output
@@ -286,16 +285,20 @@ void M6532::setPinState()
   port1.write(Controller::Two, a & 0x02);
   port1.write(Controller::Three, a & 0x04);
   port1.write(Controller::Four, a & 0x08);
+
+  if(swcha)
+  {
+    port0.controlWrite();
+    port1.controlWrite();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool M6532::save(Serializer& out) const
 {
-  const string& device = name();
-
   try
   {
-    out.putString(device);
+    out.putString(name());
 
     // Output the RAM
     out.putInt(128);
@@ -311,6 +314,7 @@ bool M6532::save(Serializer& out) const
     out.putByte((char)myDDRA);
     out.putByte((char)myDDRB);
     out.putByte((char)myOutA);
+    out.putByte((char)myOutB);
     out.putByte((char)myOutTimer[0]);
     out.putByte((char)myOutTimer[1]);
     out.putByte((char)myOutTimer[2]);
@@ -328,11 +332,9 @@ bool M6532::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool M6532::load(Serializer& in)
 {
-  const string& device = name();
-
   try
   {
-    if(in.getString() != device)
+    if(in.getString() != name())
       return false;
 
     // Input the RAM
@@ -349,6 +351,7 @@ bool M6532::load(Serializer& in)
     myDDRA = (uInt8) in.getByte();
     myDDRB = (uInt8) in.getByte();
     myOutA = (uInt8) in.getByte();
+    myOutB = (uInt8) in.getByte();
     myOutTimer[0] = (uInt8) in.getByte();
     myOutTimer[1] = (uInt8) in.getByte();
     myOutTimer[2] = (uInt8) in.getByte();
@@ -365,7 +368,8 @@ bool M6532::load(Serializer& in)
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 M6532::M6532(const M6532& c)
-  : myConsole(c.myConsole)
+  : myConsole(c.myConsole),
+    mySettings(c.mySettings)
 {
   assert(false);
 }
