@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystem.cxx 2251 2011-06-09 20:50:19Z stephena $
+// $Id: OSystem.cxx 2405 2012-03-04 19:20:29Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -203,9 +203,15 @@ bool OSystem::create()
   // Get updated paths for all configuration files
   setConfigPaths();
   ostringstream buf;
-  buf << "Base directory:       '" << myBaseDir << "'" << endl
-      << "Configuration file:   '" << myConfigFile << "'" << endl
-      << "User game properties: '" << myPropertiesFile << "'" << endl
+  buf << "Stella " << STELLA_VERSION << endl
+      << "  Features: " << myFeatures << endl
+      << "  " << myBuildInfo << endl << endl
+      << "Base directory:       '"
+      << FilesystemNode(myBaseDir).getRelativePath() << "'" << endl
+      << "Configuration file:   '"
+      << FilesystemNode(myConfigFile).getRelativePath() << "'" << endl
+      << "User game properties: '"
+      << FilesystemNode(myPropertiesFile).getRelativePath() << "'" << endl
       << endl;
   logMessage(buf.str(), 1);
 
@@ -274,9 +280,6 @@ bool OSystem::create()
   myMenu = new Menu(this);
   myCommandMenu = new CommandMenu(this);
   myLauncher = new Launcher(this);
-#ifdef DEBUGGER_SUPPORT
-  myDebugger = new Debugger(this);
-#endif
   myStateManager = new StateManager(this);
 
   // Create the sound object; the sound subsystem isn't actually
@@ -304,6 +307,16 @@ bool OSystem::create()
   return true;
 }
 
+#ifdef DEBUGGER_SUPPORT
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void OSystem::createDebugger(Console& console)
+{
+  delete myDebugger;  myDebugger = NULL;
+  myDebugger = new Debugger(*this, console);
+  myDebugger->initialize();
+}
+#endif
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystem::setConfigPaths()
 {
@@ -313,31 +326,28 @@ void OSystem::setConfigPaths()
   FilesystemNode node;
   string s;
 
-  validatePath("statedir", "state", myStateDir);
-
-  validatePath("ssdir", "snapshots", mySnapshotDir);
-
-  validatePath("eepromdir", "", myEEPROMDir);
-
-  validatePath("cfgdir", "cfg", myCfgDir);
+  validatePath(myStateDir, "statedir", myBaseDir + "statedir");
+  validatePath(mySnapshotDir, "snapdir", defaultSnapDir());
+  validatePath(myEEPROMDir, "eepromdir", myBaseDir);
+  validatePath(myCfgDir, "cfgdir", myBaseDir + "cfg");
 
   s = mySettings->getString("cheatfile");
   if(s == "") s = myBaseDir + "stella.cht";
   node = FilesystemNode(s);
   myCheatFile = node.getPath();
-  mySettings->setString("cheatfile", node.getPath(false));
+  mySettings->setString("cheatfile", node.getRelativePath());
 
   s = mySettings->getString("palettefile");
   if(s == "") s = myBaseDir + "stella.pal";
   node = FilesystemNode(s);
   myPaletteFile = node.getPath();
-  mySettings->setString("palettefile", node.getPath(false));
+  mySettings->setString("palettefile", node.getRelativePath());
 
   s = mySettings->getString("propsfile");
   if(s == "") s = myBaseDir + "stella.pro";
   node = FilesystemNode(s);
   myPropertiesFile = node.getPath();
-  mySettings->setString("propsfile", node.getPath(false));
+  mySettings->setString("propsfile", node.getRelativePath());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -443,7 +453,7 @@ FBInitStatus OSystem::createFrameBuffer()
   // Otherwise, pass the error to the parent
 fallback:
   if(fbstatus == kFailNotSupported && myFrameBuffer &&
-     myFrameBuffer->type() == kGLBuffer)
+     myFrameBuffer->type() == kDoubleBuffer)
   {
     logMessage("ERROR: OpenGL mode failed, fallback to software\n", 0);
     delete myFrameBuffer; myFrameBuffer = NULL;
@@ -464,8 +474,8 @@ fallback:
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystem::createSound()
 {
-  delete mySound;  mySound = NULL;
-  mySound = MediaFactory::createAudio(this);
+  if(!mySound)
+    mySound = MediaFactory::createAudio(this);
 #ifndef SOUND_SUPPORT
   mySettings->setBool("sound", false);
 #endif
@@ -510,7 +520,6 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
   #ifdef CHEATCODE_SUPPORT
     myCheatManager->loadCheats(myRomMD5);
   #endif
-    bool audiofirst = mySettings->getBool("audiofirst");
     //////////////////////////////////////////////////////////////////////////
     // For some reason, ATI video drivers for OpenGL in Win32 cause problems
     // if the sound isn't initialized before the video
@@ -520,15 +529,15 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
     // http://www.atariage.com/forums/index.php?showtopic=126090&view=findpost&p=1648693
     // Hopefully it won't break anything else
     //////////////////////////////////////////////////////////////////////////
-    if(audiofirst)  myConsole->initializeAudio();
+    myConsole->initializeAudio();
     myEventHandler->reset(EventHandler::S_EMULATE);
+    myEventHandler->setMouseControllerMode(mySettings->getString("mcontrol"));
     if(createFrameBuffer() != kSuccess)  // Takes care of initializeVideo()
     {
       logMessage("ERROR: Couldn't create framebuffer for console\n", 0);
       myEventHandler->reset(EventHandler::S_LAUNCHER);
       return false;
     }
-    if(!audiofirst)  myConsole->initializeAudio();
 
     if(showmessage)
     {
@@ -538,7 +547,7 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
         myFrameBuffer->showMessage("Multicart " + type + ", loading ROM" + id);
     }
     buf << "Game console created:" << endl
-        << "  ROM file: " << myRomFile << endl << endl
+        << "  ROM file: " << FilesystemNode(myRomFile).getRelativePath() << endl << endl
         << getROMInfo(myConsole) << endl;
     logMessage(buf.str(), 1);
 
@@ -554,7 +563,7 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
     if(mySettings->getBool("holdselect"))
       myEventHandler->handleEvent(Event::ConsoleSelect, 1);
     if(mySettings->getBool("holdbutton0"))
-      myEventHandler->handleEvent(Event::JoystickZeroFire1, 1);
+      myEventHandler->handleEvent(Event::JoystickZeroFire, 1);
 
     return true;
   }
@@ -586,12 +595,15 @@ void OSystem::deleteConsole()
     logMessage(buf.str(), 1);
 
     delete myConsole;  myConsole = NULL;
+    delete myDebugger; myDebugger = NULL;
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool OSystem::createLauncher()
+bool OSystem::createLauncher(const string& startdir)
 {
+  mySettings->setString("tmpromdir", startdir);
+
   myEventHandler->reset(EventHandler::S_LAUNCHER);
   if(createFrameBuffer() != kSuccess)
   {
@@ -646,9 +658,9 @@ void OSystem::logMessage(const string& message, uInt8 level)
     cout << message << flush;
     myLogMessages += message;
   }
-  else if(level <= (uInt8)mySettings->getInt("showinfo"))
+  else if(level <= (uInt8)mySettings->getInt("loglevel"))
   {
-    if(1) // TODO - messages should be output to console
+    if(mySettings->getBool("logtoconsole"))
       cout << message << flush;
     myLogMessages += message;
   }
@@ -707,6 +719,7 @@ Console* OSystem::openConsole(const string& romfile, string& md5,
     s = mySettings->getString("bc");
     if(s != "") { props.set(Controller_Left, s); props.set(Controller_Right, s); }
     CMDLINE_PROPS_UPDATE("cp", Controller_SwapPaddles);
+    CMDLINE_PROPS_UPDATE("ma", Controller_MouseAxis);
     CMDLINE_PROPS_UPDATE("format", Display_Format);
     CMDLINE_PROPS_UPDATE("ystart", Display_YStart);
     CMDLINE_PROPS_UPDATE("height", Display_Height);
@@ -869,56 +882,35 @@ void OSystem::resetLoopTiming()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::validatePath(const string& setting, const string& partialpath,
-                           string& fullpath)
+void OSystem::validatePath(string& path, const string& setting,
+                           const string& defaultpath)
 {
-  const string& s = mySettings->getString(setting) != "" ?
-    mySettings->getString(setting) : myBaseDir + partialpath;
+  const string& s = mySettings->getString(setting) == "" ? defaultpath :
+                    mySettings->getString(setting);
   FilesystemNode node(s);
   if(!node.isDirectory())
   {
     AbstractFilesystemNode::makeDir(s);
     node = FilesystemNode(node.getPath());
   }
-  fullpath = node.getPath();
-  mySettings->setString(setting, node.getPath(false));
+  path = node.getPath();
+  mySettings->setString(setting, node.getRelativePath());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystem::setDefaultJoymap(Event::Type event, EventMode mode)
 {
-#define SET_DEFAULT_BTN(sdb_event, sdb_mode, sdb_stick, sdb_button, sdb_cmp_event) \
-  if(eraseAll || sdb_cmp_event == sdb_event) \
-    myEventHandler->setDefaultJoyMapping(sdb_event, sdb_mode, sdb_stick, sdb_button);
-
-  bool eraseAll = (event == Event::NoType);
-  switch(mode)
-  {
-    case kEmulationMode:  // Default emulation events
-      // Left joystick (assume joystick zero, button zero)
-      SET_DEFAULT_BTN(Event::JoystickZeroFire1, mode, 0, 0, event);
-      // Right joystick (assume joystick one, button zero)
-      SET_DEFAULT_BTN(Event::JoystickOneFire1, mode, 1, 0, event);
-      break;
-
-    case kMenuMode:  // Default menu/UI events
-      // Left joystick (assume joystick zero, button zero)
-      SET_DEFAULT_BTN(Event::UISelect, mode, 0, 0, event);
-      // Right joystick (assume joystick one, button zero)
-      SET_DEFAULT_BTN(Event::UISelect, mode, 1, 0, event);
-      break;
-
-    default:
-      break;
-  }
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::setDefaultJoyAxisMap(Event::Type event, EventMode mode)
-{
 #define SET_DEFAULT_AXIS(sda_event, sda_mode, sda_stick, sda_axis, sda_val, sda_cmp_event) \
   if(eraseAll || sda_cmp_event == sda_event) \
-    myEventHandler->setDefaultJoyAxisMapping(sda_event, sda_mode, sda_stick, sda_axis, sda_val);
+    myEventHandler->addJoyAxisMapping(sda_event, sda_mode, sda_stick, sda_axis, sda_val, false);
+
+#define SET_DEFAULT_BTN(sdb_event, sdb_mode, sdb_stick, sdb_button, sdb_cmp_event) \
+  if(eraseAll || sdb_cmp_event == sdb_event) \
+    myEventHandler->addJoyButtonMapping(sdb_event, sdb_mode, sdb_stick, sdb_button, false);
+
+#define SET_DEFAULT_HAT(sdh_event, sdh_mode, sdh_stick, sdh_hat, sdh_dir, sdh_cmp_event) \
+  if(eraseAll || sdh_cmp_event == sdh_event) \
+    myEventHandler->addJoyHatMapping(sdh_event, sdh_mode, sdh_stick, sdh_hat, sdh_dir, false);
 
   bool eraseAll = (event == Event::NoType);
   switch(mode)
@@ -936,6 +928,19 @@ void OSystem::setDefaultJoyAxisMap(Event::Type event, EventMode mode)
       // Right joystick left/right directions (assume joystick one)
       SET_DEFAULT_AXIS(Event::JoystickOneUp, mode, 1, 1, 0, event);
       SET_DEFAULT_AXIS(Event::JoystickOneDown, mode, 1, 1, 1, event);
+
+      // Left joystick (assume joystick zero, button zero)
+      SET_DEFAULT_BTN(Event::JoystickZeroFire, mode, 0, 0, event);
+      // Right joystick (assume joystick one, button zero)
+      SET_DEFAULT_BTN(Event::JoystickOneFire, mode, 1, 0, event);
+
+      // Left joystick left/right directions (assume joystick zero and hat 0)
+      SET_DEFAULT_HAT(Event::JoystickZeroLeft, mode, 0, 0, EVENT_HATLEFT, event);
+      SET_DEFAULT_HAT(Event::JoystickZeroRight, mode, 0, 0, EVENT_HATRIGHT, event);
+      // Left joystick up/down directions (assume joystick zero and hat 0)
+      SET_DEFAULT_HAT(Event::JoystickZeroUp, mode, 0, 0, EVENT_HATUP, event);
+      SET_DEFAULT_HAT(Event::JoystickZeroDown, mode, 0, 0, EVENT_HATDOWN, event);
+
       break;
 
     case kMenuMode:  // Default menu/UI events
@@ -943,37 +948,17 @@ void OSystem::setDefaultJoyAxisMap(Event::Type event, EventMode mode)
       SET_DEFAULT_AXIS(Event::UIRight, mode, 0, 0, 1, event);
       SET_DEFAULT_AXIS(Event::UIUp, mode, 0, 1, 0, event);
       SET_DEFAULT_AXIS(Event::UIDown, mode, 0, 1, 1, event);
-      break;
 
-    default:
-      break;
-  }
-}
+      // Left joystick (assume joystick zero, button zero)
+      SET_DEFAULT_BTN(Event::UISelect, mode, 0, 0, event);
+      // Right joystick (assume joystick one, button zero)
+      SET_DEFAULT_BTN(Event::UISelect, mode, 1, 0, event);
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystem::setDefaultJoyHatMap(Event::Type event, EventMode mode)
-{
-#define SET_DEFAULT_HAT(sdh_event, sdh_mode, sdh_stick, sdh_hat, sdh_dir, sdh_cmp_event) \
-  if(eraseAll || sdh_cmp_event == sdh_event) \
-    myEventHandler->setDefaultJoyHatMapping(sdh_event, sdh_mode, sdh_stick, sdh_hat, sdh_dir);
-
-  bool eraseAll = (event == Event::NoType);
-  switch(mode)
-  {
-    case kEmulationMode:  // Default emulation events
-      // Left joystick left/right directions (assume joystick zero and hat 0)
-      SET_DEFAULT_HAT(Event::JoystickZeroLeft, mode, 0, 0, EVENT_HATLEFT, event);
-      SET_DEFAULT_HAT(Event::JoystickZeroRight, mode, 0, 0, EVENT_HATRIGHT, event);
-      // Left joystick up/down directions (assume joystick zero and hat 0)
-      SET_DEFAULT_HAT(Event::JoystickZeroUp, mode, 0, 0, EVENT_HATUP, event);
-      SET_DEFAULT_HAT(Event::JoystickZeroDown, mode, 0, 0, EVENT_HATDOWN, event);
-      break;
-
-    case kMenuMode:  // Default menu/UI events
       SET_DEFAULT_HAT(Event::UILeft, mode, 0, 0, EVENT_HATLEFT, event);
       SET_DEFAULT_HAT(Event::UIRight, mode, 0, 0, EVENT_HATRIGHT, event);
       SET_DEFAULT_HAT(Event::UIUp, mode, 0, 0, EVENT_HATUP, event);
       SET_DEFAULT_HAT(Event::UIDown, mode, 0, 0, EVENT_HATDOWN, event);
+
       break;
 
     default:
@@ -986,14 +971,6 @@ void OSystem::pollEvent()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool OSystem::joyButtonHandled(int button)
-{
-  // Since we don't do any platform-specific event polling,
-  // no button is ever handled at this level
-  return false;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystem::stateChanged(EventHandler::State state)
 {
