@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Paddles.cxx 2296 2011-12-26 21:40:28Z stephena $
+// $Id: Paddles.cxx 2405 2012-03-04 19:20:29Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -24,7 +24,7 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Paddles::Paddles(Jack jack, const Event& event, const System& system,
-                 bool swapport, bool swapaxis, bool swapdir)
+                 bool swappaddle, bool swapaxis, bool swapdir)
   : Controller(jack, event, system, Controller::Paddles),
     myMPaddleID(-1),
     myMPaddleIDX(-1),
@@ -46,15 +46,15 @@ Paddles::Paddles(Jack jack, const Event& event, const System& system,
   // Consider whether this is the left or right port
   if(myJack == Left)
   {
-    if(!swapport)  // First paddle is 0, second is 1
+    if(!swappaddle)  // First paddle is 0, second is 1
     {
       // These aren't affected by changes in axis orientation
       myP0AxisValue  = Event::SALeftAxis0Value;
       myP1AxisValue  = Event::SALeftAxis1Value;
       myP0FireEvent1 = Event::PaddleZeroFire;
-      myP0FireEvent2 = Event::JoystickZeroFire1;
+      myP0FireEvent2 = Event::JoystickZeroFire;
       myP1FireEvent1 = Event::PaddleOneFire;
-      myP1FireEvent2 = Event::JoystickZeroFire3;
+      myP1FireEvent2 = Event::JoystickZeroFire9;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
@@ -88,9 +88,9 @@ Paddles::Paddles(Jack jack, const Event& event, const System& system,
       myP0AxisValue  = Event::SALeftAxis1Value;
       myP1AxisValue  = Event::SALeftAxis0Value;
       myP0FireEvent1 = Event::PaddleOneFire;
-      myP0FireEvent2 = Event::JoystickZeroFire3;
+      myP0FireEvent2 = Event::JoystickZeroFire9;
       myP1FireEvent1 = Event::PaddleZeroFire;
-      myP1FireEvent2 = Event::JoystickZeroFire1;
+      myP1FireEvent2 = Event::JoystickZeroFire;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
@@ -121,15 +121,15 @@ Paddles::Paddles(Jack jack, const Event& event, const System& system,
   }
   else    // Jack is right port
   {
-    if(!swapport)  // First paddle is 2, second is 3
+    if(!swappaddle)  // First paddle is 2, second is 3
     {
       // These aren't affected by changes in axis orientation
       myP0AxisValue  = Event::SARightAxis0Value;
       myP1AxisValue  = Event::SARightAxis1Value;
       myP0FireEvent1 = Event::PaddleTwoFire;
-      myP0FireEvent2 = Event::JoystickOneFire1;
+      myP0FireEvent2 = Event::JoystickOneFire;
       myP1FireEvent1 = Event::PaddleThreeFire;
-      myP1FireEvent2 = Event::JoystickOneFire3;
+      myP1FireEvent2 = Event::JoystickOneFire9;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
@@ -163,9 +163,9 @@ Paddles::Paddles(Jack jack, const Event& event, const System& system,
       myP0AxisValue  = Event::SARightAxis1Value;
       myP1AxisValue  = Event::SARightAxis0Value;
       myP0FireEvent1 = Event::PaddleThreeFire;
-      myP0FireEvent2 = Event::JoystickOneFire3;
+      myP0FireEvent2 = Event::JoystickOneFire9;
       myP1FireEvent1 = Event::PaddleTwoFire;
-      myP1FireEvent2 = Event::JoystickOneFire1;
+      myP1FireEvent2 = Event::JoystickOneFire;
 
       // Direction of movement is swapped
       // That is, moving in a certain direction on an axis can
@@ -293,7 +293,8 @@ void Paddles::update()
       myCharge[myMPaddleID] = TRIGMIN;
     if(myCharge[myMPaddleID] > TRIGMAX)
       myCharge[myMPaddleID] = TRIGMAX;
-    if(myEvent.get(Event::MouseButtonValue))
+    if(myEvent.get(Event::MouseButtonLeftValue) ||
+       myEvent.get(Event::MouseButtonRightValue))
       myDigitalPinState[ourButtonPin[myMPaddleID]] = false;
   }
   else
@@ -308,6 +309,8 @@ void Paddles::update()
         myCharge[myMPaddleIDX] = TRIGMIN;
       if(myCharge[myMPaddleIDX] > TRIGMAX)
         myCharge[myMPaddleIDX] = TRIGMAX;
+      if(myEvent.get(Event::MouseButtonLeftValue))
+        myDigitalPinState[ourButtonPin[myMPaddleIDX]] = false;
     }
     if(myMPaddleIDY > -1)
     {
@@ -317,6 +320,8 @@ void Paddles::update()
         myCharge[myMPaddleIDY] = TRIGMIN;
       if(myCharge[myMPaddleIDY] > TRIGMAX)
         myCharge[myMPaddleIDY] = TRIGMAX;
+      if(myEvent.get(Event::MouseButtonRightValue))
+        myDigitalPinState[ourButtonPin[myMPaddleIDY]] = false;
     }
   }
 
@@ -377,11 +382,12 @@ void Paddles::update()
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Paddles::setMouseControl(
-    MouseAxisControl xaxis, MouseAxisControl yaxis, int ctrlID)
+    MouseControl::Axis xaxis, MouseControl::Axis yaxis, int ctrlID)
 {
-  // In 'automatic' mode, both axes on the mouse map to a single paddle
+  // In 'automatic' mode, both axes on the mouse map to a single paddle,
+  // and the paddle axis and direction settings are taken into account
   // This overrides any other mode
-  if(xaxis == Controller::Automatic || yaxis == Controller::Automatic)
+  if(xaxis == MouseControl::Automatic || yaxis == MouseControl::Automatic)
   {
     myMPaddleID = ((myJack == Left && (ctrlID == 0 || ctrlID == 1)) ||
                    (myJack == Right && (ctrlID == 2 || ctrlID == 3))
@@ -397,30 +403,30 @@ void Paddles::setMouseControl(
     {
       switch(xaxis)
       {
-        case Controller::Paddle0:  myMPaddleIDX = 0;  break;
-        case Controller::Paddle1:  myMPaddleIDX = 1;  break;
-        default:                   myMPaddleIDX = -1; break;
+        case MouseControl::Paddle0:  myMPaddleIDX = 0;  break;
+        case MouseControl::Paddle1:  myMPaddleIDX = 1;  break;
+        default:                     myMPaddleIDX = -1; break;
       }
       switch(yaxis)
       {
-        case Controller::Paddle0:  myMPaddleIDY = 0;  break;
-        case Controller::Paddle1:  myMPaddleIDY = 1;  break;
-        default:                   myMPaddleIDY = -1; break;
+        case MouseControl::Paddle0:  myMPaddleIDY = 0;  break;
+        case MouseControl::Paddle1:  myMPaddleIDY = 1;  break;
+        default:                     myMPaddleIDY = -1; break;
       }
     }
     else  // myJack == Right
     {
       switch(xaxis)
       {
-        case Controller::Paddle2:  myMPaddleIDX = 0;  break;
-        case Controller::Paddle3:  myMPaddleIDX = 1;  break;
-        default:                   myMPaddleIDX = -1; break;
+        case MouseControl::Paddle2:  myMPaddleIDX = 0;  break;
+        case MouseControl::Paddle3:  myMPaddleIDX = 1;  break;
+        default:                     myMPaddleIDX = -1; break;
       }
       switch(yaxis)
       {
-        case Controller::Paddle2:  myMPaddleIDY = 0;  break;
-        case Controller::Paddle3:  myMPaddleIDY = 1;  break;
-        default:                   myMPaddleIDY = -1; break;
+        case MouseControl::Paddle2:  myMPaddleIDY = 0;  break;
+        case MouseControl::Paddle3:  myMPaddleIDY = 1;  break;
+        default:                     myMPaddleIDY = -1; break;
       }
     }
   }
