@@ -8,13 +8,13 @@
 // MM     MM 66  66 55  55 00  00 22
 // MM     MM  6666   5555   0000  222222
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: System.cxx 2232 2011-05-24 16:04:48Z stephena $
+// $Id: System.cxx 2343 2012-01-08 16:55:10Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -311,6 +311,7 @@ bool System::save(Serializer& out) const
   {
     out.putString(name());
     out.putInt(myCycles);
+    out.putByte(myDataBusState);
 
     if(!myM6502->save(out))
       return false;
@@ -338,6 +339,7 @@ bool System::load(Serializer& in)
       return false;
 
     myCycles = (uInt32) in.getInt();
+    myDataBusState = (uInt8) in.getByte();
 
     // Next, load state for the CPU
     if(!myM6502->load(in))
