@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TIA.cxx 2280 2011-11-19 18:28:17Z stephena $
+// $Id: TIA.cxx 2359 2012-01-17 22:20:20Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -50,6 +50,7 @@ TIA::TIA(Console& console, Sound& sound, Settings& settings)
     myFrameYStart(34),
     myFrameHeight(210),
     myMaximumNumberOfScanlines(262),
+    myStartScanline(0),
     myColorLossEnabled(false),
     myPartialFrameFlag(false),
     myAutoFrameEnabled(false),
@@ -62,54 +63,8 @@ TIA::TIA(Console& console, Sound& sound, Settings& settings)
   // Make sure all TIA bits are enabled
   enableBits(true);
 
-  for(uInt16 x = 0; x < 2; ++x)
-  {
-    for(uInt16 enabled = 0; enabled < 256; ++enabled)
-    {
-      if(enabled & PriorityBit)
-      {
-        // Priority from highest to lowest:
-        //   PF/BL => P0/M0 => P1/M1 => BK
-        uInt8 color = _BK;
-
-        if((enabled & M1Bit) != 0)
-          color = _M1;
-        if((enabled & P1Bit) != 0)
-          color = _P1;
-        if((enabled & M0Bit) != 0)
-          color = _M0;
-        if((enabled & P0Bit) != 0)
-          color = _P0;
-        if((enabled & BLBit) != 0)
-          color = _BL;
-        if((enabled & PFBit) != 0)
-          color = _PF;  // NOTE: Playfield has priority so ScoreBit isn't used
-
-        myPriorityEncoder[x][enabled] = color;
-      }
-      else
-      {
-        // Priority from highest to lowest:
-        //   P0/M0 => P1/M1 => PF/BL => BK
-        uInt8 color = _BK;
-
-        if((enabled & BLBit) != 0)
-          color = _BL;
-        if((enabled & PFBit) != 0)
-          color = (enabled & ScoreBit) ? ((x == 0) ? _P0 : _P1) : _PF;
-        if((enabled & M1Bit) != 0)
-          color = _M1;
-        if((enabled & P1Bit) != 0)
-          color = _P1;
-        if((enabled & M0Bit) != 0)
-          color = _M0;
-        if((enabled & P0Bit) != 0)
-          color = _P0;
-
-        myPriorityEncoder[x][enabled] = color;
-      }
-    }
-  }
+  // Turn off debug colours (this also sets up the PriorityEncoder)
+  toggleFixedColors(0);
 
   // Compute all of the mask tables
   TIATables::computeAllTables();
@@ -137,11 +92,10 @@ void TIA::reset()
   myAllowHMOVEBlanks = true;
 
   // Some default values for the registers
-  myColorPtr = myColor;
   myVSYNC = myVBLANK = 0;
   myNUSIZ0 = myNUSIZ1 = 0;
-  myColor[_P0] = myColor[_P1] = myColor[_PF] = myColor[_BK] = 0;
-  myColor[_M0] = myColor[_M1] = myColor[_BL] = myColor[_HBLANK] = 0;
+  myColor[P0Color] = myColor[P1Color] = myColor[PFColor] = myColor[BKColor] = 0;
+  myColor[M0Color] = myColor[M1Color] = myColor[BLColor] = myColor[HBLANKColor] = 0;
 
   myPlayfieldPriorityAndScore = 0;
   myCTRLPF = 0;
@@ -188,27 +142,27 @@ void TIA::reset()
 
   if(myFramerate > 55.0)  // NTSC
   {
-    myFixedColor[_P0] = 0x30303030;
-    myFixedColor[_P1] = 0x16161616;
-    myFixedColor[_M0] = 0x38383838;
-    myFixedColor[_M1] = 0x12121212;
-    myFixedColor[_BL] = 0x7e7e7e7e;
-    myFixedColor[_PF] = 0x76767676;
-    myFixedColor[_BK] = 0x0a0a0a0a;
-    myFixedColor[_HBLANK] = 0x0e0e0e0e;
+    myFixedColor[P0Color]     = 0x30303030;
+    myFixedColor[P1Color]     = 0x16161616;
+    myFixedColor[M0Color]     = 0x38383838;
+    myFixedColor[M1Color]     = 0x12121212;
+    myFixedColor[BLColor]     = 0x7e7e7e7e;
+    myFixedColor[PFColor]     = 0x76767676;
+    myFixedColor[BKColor]     = 0x0a0a0a0a;
+    myFixedColor[HBLANKColor] = 0x0e0e0e0e;
     myColorLossEnabled = false;
     myMaximumNumberOfScanlines = 290;
   }
   else
   {
-    myFixedColor[_P0] = 0x62626262;
-    myFixedColor[_P1] = 0x26262626;
-    myFixedColor[_M0] = 0x68686868;
-    myFixedColor[_M1] = 0x2e2e2e2e;
-    myFixedColor[_BL] = 0xdededede;
-    myFixedColor[_PF] = 0xd8d8d8d8;
-    myFixedColor[_BK] = 0x1c1c1c1c;
-    myFixedColor[_HBLANK] = 0x0e0e0e0e;
+    myFixedColor[P0Color]     = 0x62626262;
+    myFixedColor[P1Color]     = 0x26262626;
+    myFixedColor[M0Color]     = 0x68686868;
+    myFixedColor[M1Color]     = 0x2e2e2e2e;
+    myFixedColor[BLColor]     = 0xdededede;
+    myFixedColor[PFColor]     = 0xd8d8d8d8;
+    myFixedColor[BKColor]     = 0x1c1c1c1c;
+    myFixedColor[HBLANKColor] = 0x0e0e0e0e;
     myColorLossEnabled = mySettings.getBool("colorloss");
     myMaximumNumberOfScanlines = 342;
   }
@@ -221,6 +175,7 @@ void TIA::reset()
   myCurrentPFMask = TIATables::PFMask[0];
 
   // Recalculate the size of the display
+  toggleFixedColors(0);
   frameReset();
 }
 
@@ -330,13 +285,13 @@ bool TIA::save(Serializer& out) const
     out.putByte((char)myNUSIZ0);
     out.putByte((char)myNUSIZ1);
 
-    out.putInt(myColor[_P0]);
-    out.putInt(myColor[_P1]);
-    out.putInt(myColor[_PF]);
-    out.putInt(myColor[_BK]);
-    out.putInt(myColor[_M0]);
-    out.putInt(myColor[_M1]);
-    out.putInt(myColor[_BL]);
+    out.putInt(myColor[P0Color]);
+    out.putInt(myColor[P1Color]);
+    out.putInt(myColor[PFColor]);
+    out.putInt(myColor[BKColor]);
+    out.putInt(myColor[M0Color]);
+    out.putInt(myColor[M1Color]);
+    out.putInt(myColor[BLColor]);
 
     out.putByte((char)myCTRLPF);
     out.putByte((char)myPlayfieldPriorityAndScore);
@@ -439,13 +394,13 @@ bool TIA::load(Serializer& in)
     myNUSIZ0 = (uInt8) in.getByte();
     myNUSIZ1 = (uInt8) in.getByte();
 
-    myColor[_P0] = (uInt32) in.getInt();
-    myColor[_P1] = (uInt32) in.getInt();
-    myColor[_PF] = (uInt32) in.getInt();
-    myColor[_BK] = (uInt32) in.getInt();
-    myColor[_M0] = (uInt32) in.getInt();
-    myColor[_M1] = (uInt32) in.getInt();
-    myColor[_BL] = (uInt32) in.getInt();
+    myColor[P0Color] = (uInt32) in.getInt();
+    myColor[P1Color] = (uInt32) in.getInt();
+    myColor[PFColor] = (uInt32) in.getInt();
+    myColor[BKColor] = (uInt32) in.getInt();
+    myColor[M0Color] = (uInt32) in.getInt();
+    myColor[M1Color] = (uInt32) in.getInt();
+    myColor[BLColor] = (uInt32) in.getInt();
 
     myCTRLPF = (uInt8) in.getByte();
     myPlayfieldPriorityAndScore = (uInt8) in.getByte();
@@ -515,8 +470,8 @@ bool TIA::load(Serializer& in)
 
     // Reset TIA bits to be on
     enableBits(true);
+    toggleFixedColors(0);
     myAllowHMOVEBlanks = true;
-    myColorPtr = myColor;
   }
   catch(const char* msg)
   {
@@ -629,26 +584,26 @@ inline void TIA::startFrame()
   {
     if(myScanlineCountForLastFrame & 0x01)
     {
-      myColor[_P0] |= 0x01010101;
-      myColor[_P1] |= 0x01010101;
-      myColor[_PF] |= 0x01010101;
-      myColor[_BK] |= 0x01010101;
-      myColor[_M0] |= 0x01010101;
-      myColor[_M1] |= 0x01010101;
-      myColor[_BL] |= 0x01010101;
+      myColor[P0Color] |= 0x01010101;
+      myColor[P1Color] |= 0x01010101;
+      myColor[PFColor] |= 0x01010101;
+      myColor[BKColor] |= 0x01010101;
+      myColor[M0Color] |= 0x01010101;
+      myColor[M1Color] |= 0x01010101;
+      myColor[BLColor] |= 0x01010101;
     }
     else
     {
-      myColor[_P0] &= 0xfefefefe;
-      myColor[_P1] &= 0xfefefefe;
-      myColor[_PF] &= 0xfefefefe;
-      myColor[_BK] &= 0xfefefefe;
-      myColor[_M0] &= 0xfefefefe;
-      myColor[_M1] &= 0xfefefefe;
-      myColor[_BL] &= 0xfefefefe;
+      myColor[P0Color] &= 0xfefefefe;
+      myColor[P1Color] &= 0xfefefefe;
+      myColor[PFColor] &= 0xfefefefe;
+      myColor[BKColor] &= 0xfefefefe;
+      myColor[M0Color] &= 0xfefefefe;
+      myColor[M1Color] &= 0xfefefefe;
+      myColor[BLColor] &= 0xfefefefe;
     }
-  }   
-  myStartScanline = 0x7FFFFFFF;
+  }
+  myStartScanline = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -806,6 +761,58 @@ bool TIA::toggleFixedColors(uInt8 mode)
   if(on)  myColorPtr = myFixedColor;
   else    myColorPtr = myColor;
 
+  // Set PriorityEncoder
+  // This needs to be done here, since toggling debug colours also changes
+  // how colours are interpreted in PF 'score' mode
+  for(uInt16 x = 0; x < 2; ++x)
+  {
+    for(uInt16 enabled = 0; enabled < 256; ++enabled)
+    {
+      if(enabled & PriorityBit)
+      {
+        // Priority from highest to lowest:
+        //   PF/BL => P0/M0 => P1/M1 => BK
+        uInt8 color = BKColor;
+
+        if((enabled & M1Bit) != 0)
+          color = M1Color;
+        if((enabled & P1Bit) != 0)
+          color = P1Color;
+        if((enabled & M0Bit) != 0)
+          color = M0Color;
+        if((enabled & P0Bit) != 0)
+          color = P0Color;
+        if((enabled & BLBit) != 0)
+          color = BLColor;
+        if((enabled & PFBit) != 0)
+          color = PFColor;  // NOTE: Playfield has priority so ScoreBit isn't used
+
+        myPriorityEncoder[x][enabled] = color;
+      }
+      else
+      {
+        // Priority from highest to lowest:
+        //   P0/M0 => P1/M1 => PF/BL => BK
+        uInt8 color = BKColor;
+
+        if((enabled & BLBit) != 0)
+          color = BLColor;
+        if((enabled & PFBit) != 0)
+          color = (!on && (enabled & ScoreBit)) ? ((x == 0) ? P0Color : P1Color) : PFColor;
+        if((enabled & M1Bit) != 0)
+          color = M1Color;
+        if((enabled & P1Bit) != 0)
+          color = P1Color;
+        if((enabled & M0Bit) != 0)
+          color = M0Color;
+        if((enabled & P0Bit) != 0)
+          color = P0Color;
+
+        myPriorityEncoder[x][enabled] = color;
+      }
+    }
+  }
+
   return on;
 }
 
@@ -1094,7 +1101,7 @@ void TIA::updateFrame(Int32 clock)
         (clocksFromStartOfScanLine < (HBLANK + 8)))
     {
       Int32 blanks = (HBLANK + 8) - clocksFromStartOfScanLine;
-      memset(oldFramePointer, myColorPtr[_HBLANK], blanks);
+      memset(oldFramePointer, myColorPtr[HBLANKColor], blanks);
 
       if((clocksToUpdate + clocksFromStartOfScanLine) >= (HBLANK + 8))
         myHMOVEBlankEnabled = false;
@@ -1321,16 +1328,14 @@ bool TIA::poke(uInt16 addr, uInt8 value)
         myDumpDisabledCycle = mySystem->cycles();
       }
 
-      // Are the latches for I4 and I5 being set?
+      // Are the latches for I4 and I5 being reset?
       if (!(myVBLANK & 0x40))
         myINPT4 = myINPT5 = 0x80;
 
-#if 0 // TODO - this isn't yet complete
       // Check for the first scanline at which VBLANK is disabled.
       // Usually, this will be the first scanline to start drawing.
-      if(myStartScanline == 0x7FFFFFFF && !(value & 0x10))
+      if(myStartScanline == 0 && !(value & 0x10))
         myStartScanline = scanlines();
-#endif
 
       myVBLANK = value;
       break;
@@ -1384,7 +1389,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       {
         color |= 0x01;
       }
-      myColor[_P0] = myColor[_M0] =
+      myColor[P0Color] = myColor[M0Color] =
           (((((color << 8) | color) << 8) | color) << 8) | color;
       break;
     }
@@ -1396,7 +1401,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       {
         color |= 0x01;
       }
-      myColor[_P1] = myColor[_M1] =
+      myColor[P1Color] = myColor[M1Color] =
           (((((color << 8) | color) << 8) | color) << 8) | color;
       break;
     }
@@ -1408,7 +1413,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       {
         color |= 0x01;
       }
-      myColor[_PF] = myColor[_BL] =
+      myColor[PFColor] = myColor[BLColor] =
           (((((color << 8) | color) << 8) | color) << 8) | color;
       break;
     }
@@ -1420,7 +1425,7 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       {
         color |= 0x01;
       }
-      myColor[_BK] = (((((color << 8) | color) << 8) | color) << 8) | color;
+      myColor[BKColor] = (((((color << 8) | color) << 8) | color) << 8) | color;
       break;
     }
 
