@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: GameInfoDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: GameInfoDialog.cxx 2415 2012-03-15 23:40:36Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -43,6 +43,7 @@ GameInfoDialog::GameInfoDialog(
     myPropertiesLoaded(false),
     myDefaultsSelected(false)
 {
+  const GUI::Font& ifont = instance().infoFont();
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
             fontHeight   = font.getFontHeight(),
@@ -66,7 +67,7 @@ GameInfoDialog::GameInfoDialog(
   // The tab widget
   xpos = 2; ypos = vBorder;
   myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos,
-                        _h - buttonHeight - 2*fontHeight - 20);
+                        _h - buttonHeight - fontHeight - ifont.getLineHeight() - 20);
   addTabWidget(myTab);
   addFocusWidget(myTab);
 
@@ -132,7 +133,7 @@ GameInfoDialog::GameInfoDialog(
   ypos += lineHeight + 3;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Type:", kTextAlignLeft);
-  pwidth = font.getStringWidth("EFSC (64K H. Runner + ram)");
+  pwidth = font.getStringWidth("CM (SpectraVideo CompuMate)");
   items.clear();
   items.push_back("Auto-detect",          "AUTO-DETECT");
   items.push_back("0840 (8K ECONObank)",        "0840" );
@@ -148,6 +149,7 @@ GameInfoDialog::GameInfoDialog(
   items.push_back("4K (4K Atari)",              "4K"   );
   items.push_back("AR (Supercharger)",          "AR"   );
   items.push_back("CV (Commavid extra ram)",    "CV"   );
+  items.push_back("CM (SpectraVideo CompuMate)","CM"   );
   items.push_back("DPC (Pitfall II)",           "DPC"  );
   items.push_back("DPC+ (Enhanced DPC)",        "DPC+" );
   items.push_back("E0 (8K Parker Bros)",        "E0"   );
@@ -162,6 +164,7 @@ GameInfoDialog::GameInfoDialog(
   items.push_back("F8 (8K Atari)",              "F8"   );
   items.push_back("F8SC (8K Atari + ram)",      "F8SC" );
   items.push_back("FA (CBS RAM Plus)",          "FA"   );
+  items.push_back("FA2 (CBS RAM Plus 24K)",     "FA2"  );
   items.push_back("FE (8K Decathlon)",          "FE"   );
   items.push_back("MC (C. Wilkson Megacart)",   "MC"   );
   items.push_back("SB (128-256K SUPERbank)",    "SB"   );
@@ -204,7 +207,7 @@ GameInfoDialog::GameInfoDialog(
                        "TV Type:", kTextAlignLeft);
   items.clear();
   items.push_back("Color", "COLOR");
-  items.push_back("B & W", "BLACKANDWHITE");
+  items.push_back("B & W", "BW");
   myTVType = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
                              pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myTVType);
@@ -223,18 +226,23 @@ GameInfoDialog::GameInfoDialog(
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "P0 Controller:", kTextAlignLeft);
   ctrls.clear();
-  ctrls.push_back("Joystick",       "JOYSTICK"    );
-  ctrls.push_back("Paddles",        "PADDLES"     );
-  ctrls.push_back("BoosterGrip",    "BOOSTERGRIP" );
-  ctrls.push_back("Driving",        "DRIVING"     );
-  ctrls.push_back("Keyboard",       "KEYBOARD"    );
-  ctrls.push_back("CX-22 Trakball", "TRACKBALL22" );
-  ctrls.push_back("CX-80 Mouse",    "TRACKBALL80" );
-  ctrls.push_back("AmigaMouse",     "AMIGAMOUSE"  );
-  ctrls.push_back("AtariVox",       "ATARIVOX"    );
-  ctrls.push_back("SaveKey",        "SAVEKEY"     );
-  ctrls.push_back("Sega Genesis",   "GENESIS"     );
-// TODO  ctrls.push_back("KidVid",         "KIDVID"      );
+  ctrls.push_back("Joystick",       "JOYSTICK"     );
+  ctrls.push_back("Paddles",        "PADDLES"      );
+  ctrls.push_back("Paddles_IAxis",  "PADDLES_IAXIS");
+  ctrls.push_back("Paddles_IDir",   "PADDLES_IDIR" );
+  ctrls.push_back("Paddles_IAxDr",  "PADDLES_IAXDR");
+  ctrls.push_back("BoosterGrip",    "BOOSTERGRIP"  );
+  ctrls.push_back("Driving",        "DRIVING"      );
+  ctrls.push_back("Keyboard",       "KEYBOARD"     );
+  ctrls.push_back("CX-22 Trakball", "TRACKBALL22"  );
+  ctrls.push_back("CX-80 Mouse",    "TRACKBALL80"  );
+  ctrls.push_back("AmigaMouse",     "AMIGAMOUSE"   );
+  ctrls.push_back("AtariVox",       "ATARIVOX"     );
+  ctrls.push_back("SaveKey",        "SAVEKEY"      );
+  ctrls.push_back("Sega Genesis",   "GENESIS"      );
+  ctrls.push_back("CompuMate",      "COMPUMATE"    );
+//  ctrls.push_back("KidVid",         "KIDVID"      );
+//  ctrls.push_back("MindLink",       "MINDLINK"     );
   myP0Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
                                    pwidth, lineHeight, ctrls, "", 0, 0);
   wid.push_back(myP0Controller);
@@ -279,6 +287,37 @@ GameInfoDialog::GameInfoDialog(
                                   pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(mySwapPaddles);
 
+  ypos += lineHeight + 8;
+  lwidth = font.getStringWidth("Mouse axis mode: ");
+  pwidth = font.getStringWidth("Specific axis");
+  items.clear();
+  items.push_back("Automatic", "auto");
+  items.push_back("Specific axis", "specific");
+  myMouseControl =
+    new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
+                   "Mouse axis mode: ", lwidth, kMCtrlChanged);
+  wid.push_back(myMouseControl);
+
+  // Mouse controller specific axis
+  lwidth = font.getStringWidth("X-Axis is: ");
+  pwidth = font.getStringWidth("Driving 0");
+  items.clear();
+  items.push_back("Paddle 0",  BSPF_toString(MouseControl::Paddle0));
+  items.push_back("Paddle 1",  BSPF_toString(MouseControl::Paddle1));
+  items.push_back("Paddle 2",  BSPF_toString(MouseControl::Paddle2));
+  items.push_back("Paddle 3",  BSPF_toString(MouseControl::Paddle3));
+  items.push_back("Driving 0", BSPF_toString(MouseControl::Driving0));
+  items.push_back("Driving 1", BSPF_toString(MouseControl::Driving1));
+
+  xpos = 45;  ypos += lineHeight + 4;
+  myMouseX = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
+               "X-Axis is: ", lwidth);
+  wid.push_back(myMouseX);
+
+  ypos += lineHeight + 4;
+  myMouseY = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
+               "Y-Axis is: ", lwidth);
+  wid.push_back(myMouseY);
 
   // Add items for tab 2
   addToFocusList(wid, tabID);
@@ -351,8 +390,8 @@ GameInfoDialog::GameInfoDialog(
   myTab->setActiveTab(0);
 
   // Add message concerning usage
-  lwidth = font.getStringWidth("(*) Changes to properties require a ROM reload");
-  new StaticTextWidget(this, font, 10, _h - buttonHeight - fontHeight - 20,
+  lwidth = ifont.getStringWidth("(*) Changes to properties require a ROM reload");
+  new StaticTextWidget(this, ifont, 10, _h - buttonHeight - fontHeight - 20,
                        lwidth, fontHeight,
                        "(*) Changes to properties require a ROM reload",
                        kTextAlignLeft);
@@ -425,6 +464,22 @@ void GameInfoDialog::loadView()
   myP0Controller->setSelected(myGameProperties.get(Controller_Left), "JOYSTICK");
   myP1Controller->setSelected(myGameProperties.get(Controller_Right), "JOYSTICK");
   mySwapPaddles->setSelected(myGameProperties.get(Controller_SwapPaddles), "NO");
+  const string& mcontrol = myGameProperties.get(Controller_MouseAxis);
+  bool autoAxis = BSPF_equalsIgnoreCase(mcontrol, "auto");
+  if(autoAxis)
+  {
+    myMouseControl->setSelected(0);
+    myMouseX->setSelected(0);
+    myMouseY->setSelected(0);
+  }
+  else
+  {
+    myMouseControl->setSelected(1);
+    myMouseX->setSelected(BSPF_toString(mcontrol[0] - '0'), "");
+    myMouseY->setSelected(BSPF_toString(mcontrol[1] - '0'), "");
+  }
+  myMouseX->setEnabled(!autoAxis);
+  myMouseY->setEnabled(!autoAxis);
 
   // Display properties
   myFormat->setSelected(myGameProperties.get(Display_Format), "AUTO-DETECT");
@@ -469,6 +524,10 @@ void GameInfoDialog::saveConfig()
   myGameProperties.set(Console_SwapPorts,
     myLeftPort->getSelectedTag() == "L" ? "NO" : "YES");
   myGameProperties.set(Controller_SwapPaddles, mySwapPaddles->getSelectedTag());
+  string mcontrol = myMouseControl->getSelectedTag();
+  if(mcontrol != "auto")
+    mcontrol = myMouseX->getSelectedTag() + myMouseY->getSelectedTag();
+  myGameProperties.set(Controller_MouseAxis, mcontrol);
 
   // Display properties
   myGameProperties.set(Display_Format, myFormat->getSelectedTag());
@@ -538,6 +597,14 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
       myPPBlendLabel->setValue(myPPBlend->getValue());
       break;
 
+    case kMCtrlChanged:
+    {
+      bool state = myMouseControl->getSelectedTag() != "auto";
+      myMouseX->setEnabled(state);
+      myMouseY->setEnabled(state);
+      break;
+    }
+
     default:
       Dialog::handleCommand(sender, cmd, data, 0);
       break;
