@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: GlobalPropsDialog.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: GlobalPropsDialog.cxx 2362 2012-01-21 13:14:48Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -44,7 +44,7 @@ GlobalPropsDialog::
             buttonHeight = font.getLineHeight() + 4;
   int xpos, ypos;
   int lwidth = font.getStringWidth("Right Difficulty: "),
-      pwidth = font.getStringWidth("EFSC (64K H. Runner + ram)");
+      pwidth = font.getStringWidth("CM (SpectraVideo CompuMate)");
   WidgetArray wid;
   StringMap items;
 
@@ -79,6 +79,7 @@ GlobalPropsDialog::
   items.push_back("4K (4K Atari)",              "4K"   );
   items.push_back("AR (Supercharger)",          "AR"   );
   items.push_back("CV (Commavid extra ram)",    "CV"   );
+  items.push_back("CM (SpectraVideo CompuMate)","CM"   );
   items.push_back("DPC (Pitfall II)",           "DPC"  );
   items.push_back("DPC+ (Enhanced DPC)",        "DPC+" );
   items.push_back("E0 (8K Parker Bros)",        "E0"   );
@@ -93,6 +94,7 @@ GlobalPropsDialog::
   items.push_back("F8 (8K Atari)",              "F8"   );
   items.push_back("F8SC (8K Atari + ram)",      "F8SC" );
   items.push_back("FA (CBS RAM Plus)",          "FA"   );
+  items.push_back("FA2 (CBS RAM Plus 24K)",     "FA2"  );
   items.push_back("FE (8K Decathlon)",          "FE"   );
   items.push_back("MC (C. Wilkson Megacart)",   "MC"   );
   items.push_back("SB (128-256K SUPERbank)",    "SB"   );
