@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: InputDialog.cxx 2296 2011-12-26 21:40:28Z stephena $
+// $Id: InputDialog.cxx 2401 2012-03-03 00:56:31Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -121,8 +121,8 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
   pwidth = font.getStringWidth("left / right");
 
   items.clear();
-  items.push_back("left / right", "leftright");
-  items.push_back("right / left", "rightleft");
+  items.push_back("left / right", "lr");
+  items.push_back("right / left", "rl");
   mySAPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
                              "Stelladaptor port order: ", lwidth);
   wid.push_back(mySAPort);
@@ -176,7 +176,7 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
   wid.push_back(myMPaddleSpeed);
 
   // Add 'allow all 4 directions' for joystick
-  xpos = 10;  ypos += lineHeight + 8;
+  xpos = 10;  ypos += lineHeight + 12;
   myAllowAll4 = new CheckboxWidget(myTab, font, xpos, ypos,
                   "Allow all 4 directions on joystick");
   wid.push_back(myAllowAll4);
@@ -191,37 +191,18 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
 #endif
 
   // Mouse is controller type
-  ypos += lineHeight + 8;
+  ypos += lineHeight + 12;
   lwidth = font.getStringWidth("Use mouse as a controller: ");
-  pwidth = font.getStringWidth("Specific axis");
+  pwidth = font.getStringWidth("Automatic");
   items.clear();
+  items.push_back("Never", "never");
   items.push_back("Automatic", "auto");
-  items.push_back("Specific axis", "paddle");
+  items.push_back("By ROM", "rom");
   myMouseControl =
     new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
-                   "Use mouse as a controller: ", lwidth, kMPCtrlChanged);
+                   "Use mouse as a controller: ", lwidth);
   wid.push_back(myMouseControl);
 
-  // Mouse controller specific axis
-  lwidth = font.getStringWidth("X-Axis is: ");
-  pwidth = font.getStringWidth("Paddle 3");
-  items.clear();
-  items.push_back("not used", BSPF_toString(Controller::NoControl));
-  items.push_back("Paddle 0", BSPF_toString(Controller::Paddle0));
-  items.push_back("Paddle 1", BSPF_toString(Controller::Paddle1));
-  items.push_back("Paddle 2", BSPF_toString(Controller::Paddle2));
-  items.push_back("Paddle 3", BSPF_toString(Controller::Paddle3));
-
-  xpos = 45;  ypos += lineHeight + 4;
-  myMouseX = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
-               "X-Axis is: ", lwidth);
-  wid.push_back(myMouseX);
-
-  ypos += lineHeight + 4;
-  myMouseY = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
-               "Y-Axis is: ", lwidth);
-  wid.push_back(myMouseY);
-
   // Add items for virtual device ports
   addToFocusList(wid, tabID);
 }
@@ -230,9 +211,8 @@ void InputDialog::addDevicePortTab(const GUI::Font& font)
 void InputDialog::loadConfig()
 {
   // Left & right ports
-  const string& sa1 = instance().settings().getString("sa1");
-  int portorder = sa1 == "right" ? 1 : 0;
-  mySAPort->setSelected(portorder);
+  const string& saport = instance().settings().getString("saport");
+  mySAPort->setSelected(BSPF_equalsIgnoreCase(saport, "rl") ? 1 : 0);
 
   // Joystick deadzone
   myDeadzone->setValue(instance().settings().getInt("joydeadzone"));
@@ -254,22 +234,8 @@ void InputDialog::loadConfig()
   myAllowAll4->setState(instance().settings().getBool("joyallow4"));
 
   // Mouse is controller type
-  const string& mcontrol = instance().settings().getString("mcontrol");
-  bool autoAxis = mcontrol == "auto";
-  if(autoAxis)
-  {
-    myMouseControl->setSelected(0);
-    myMouseX->setSelected(0);
-    myMouseY->setSelected(0);
-  }
-  else
-  {
-    myMouseControl->setSelected(1);
-    myMouseX->setSelected(BSPF_toString(mcontrol[0] - '0'), "");
-    myMouseY->setSelected(BSPF_toString(mcontrol[1] - '0'), "");
-  }
-  myMouseX->setEnabled(!autoAxis);
-  myMouseY->setEnabled(!autoAxis);
+  myMouseControl->setSelected(
+    instance().settings().getString("mcontrol"), "auto");
 
   myTab->loadConfig();
 }
@@ -278,11 +244,7 @@ void InputDialog::loadConfig()
 void InputDialog::saveConfig()
 {
   // Left & right ports
-  int sa_order = mySAPort->getSelected();
-  if(sa_order == 0)
-    instance().eventHandler().mapStelladaptors("left", "right");
-  else
-    instance().eventHandler().mapStelladaptors("right", "left");
+  instance().eventHandler().mapStelladaptors(mySAPort->getSelectedTag());
 
   // Joystick deadzone
   int deadzone = myDeadzone->getValue();
@@ -310,9 +272,7 @@ void InputDialog::saveConfig()
   instance().eventHandler().allowAllDirections(allowall4);
 
   // Mouse is controller type
-  string mcontrol = myMouseControl->getSelectedTag();
-  if(mcontrol != "auto")
-    mcontrol = myMouseX->getSelectedTag() + myMouseY->getSelectedTag();
+  const string& mcontrol = myMouseControl->getSelectedTag();
   instance().settings().setString("mcontrol", mcontrol);
   instance().eventHandler().setMouseControllerMode(mcontrol);
 }
@@ -333,7 +293,7 @@ void InputDialog::setDefaults()
     case 2:  // Virtual devices
     {
       // Left & right ports
-      mySAPort->setSelected("leftright", "leftright");
+      mySAPort->setSelected("lr", "lr");
 
       // Joystick deadzone
       myDeadzone->setValue(0);
@@ -355,11 +315,7 @@ void InputDialog::setDefaults()
       myAllowAll4->setState(false);
 
       // Mouse is controller type
-      myMouseControl->setSelected(0);
-      myMouseX->setSelected(0);
-      myMouseY->setSelected(0);
-      myMouseX->setEnabled(false);
-      myMouseY->setEnabled(false);
+      myMouseControl->setSelected(1);
       break;
     }
 
@@ -371,15 +327,15 @@ void InputDialog::setDefaults()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void InputDialog::handleKeyDown(int ascii, int keycode, int modifiers)
+void InputDialog::handleKeyDown(StellaKey key, StellaMod mod, char ascii)
 {
   // Remap key events in remap mode, otherwise pass to parent dialog
   if(myEmulEventMapper->remapMode())
-    myEmulEventMapper->handleKeyDown(ascii, keycode, modifiers);
+    myEmulEventMapper->handleKeyDown(key, mod, ascii);
   else if(myMenuEventMapper->remapMode())
-    myMenuEventMapper->handleKeyDown(ascii, keycode, modifiers);
+    myMenuEventMapper->handleKeyDown(key, mod, ascii);
   else
-    Dialog::handleKeyDown(ascii, keycode, modifiers);
+    Dialog::handleKeyDown(key, mod, ascii);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -450,14 +406,6 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       myMPaddleLabel->setValue(myMPaddleSpeed->getValue());
       break;
 
-    case kMPCtrlChanged:
-    {
-      bool state = myMouseControl->getSelectedTag() != "auto";
-      myMouseX->setEnabled(state);
-      myMouseY->setEnabled(state);
-      break;
-    }
-
     default:
       Dialog::handleCommand(sender, cmd, data, 0);
   }
