@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: LauncherDialog.cxx 2292 2011-12-22 21:29:24Z stephena $
+// $Id: LauncherDialog.cxx 2356 2012-01-14 22:00:54Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -233,7 +233,9 @@ const string& LauncherDialog::selectedRomMD5()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::loadConfig()
 {
-  const string& romdir = instance().settings().getString("romdir");
+  const string& tmpromdir = instance().settings().getString("tmpromdir");
+  const string& romdir = tmpromdir != "" ? tmpromdir :
+      instance().settings().getString("romdir");
 
   // When romdir hasn't been set, it probably indicates that this is the first
   // time running Stella; in this case, we should prompt the user
@@ -476,14 +478,14 @@ bool LauncherDialog::matchPattern(const string& s, const string& pattern) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void LauncherDialog::handleKeyDown(int ascii, int keycode, int modifiers)
+void LauncherDialog::handleKeyDown(StellaKey key, StellaMod mod, char ascii)
 {
   // Grab the key before passing it to the actual dialog and check for
   // Control-R (reload ROM listing)
-  if(instance().eventHandler().kbdControl(modifiers) && keycode == 'r')
+  if(instance().eventHandler().kbdControl(mod) && key == KBDK_r)
     updateListing();
   else
-    Dialog::handleKeyDown(ascii, keycode, modifiers);
+    Dialog::handleKeyDown(key, mod, ascii);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
