@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: VideoDialog.cxx 2312 2011-12-29 19:10:57Z stephena $
+// $Id: VideoDialog.cxx 2403 2012-03-03 18:26:28Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -201,12 +201,6 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
   wid.push_back(myGLStretchCheckbox);
   ypos += lineHeight + 4;
 
-  // Use VBO in OpenGL
-  myUseVBOCheckbox = new CheckboxWidget(myTab, font, xpos, ypos,
-                                        "GL VBO");
-  wid.push_back(myUseVBOCheckbox);
-  ypos += lineHeight + 4;
-
   // Use sync to vblank in OpenGL
   myUseVSyncCheckbox = new CheckboxWidget(myTab, font, xpos, ypos,
                                           "GL VSync");
@@ -275,7 +269,6 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
   myPAspectRatioSlider->clearFlags(WIDGET_ENABLED);
   myPAspectRatioLabel->clearFlags(WIDGET_ENABLED);
   myGLStretchCheckbox->clearFlags(WIDGET_ENABLED);
-  myUseVBOCheckbox->clearFlags(WIDGET_ENABLED);
   myUseVSyncCheckbox->clearFlags(WIDGET_ENABLED);
 #endif
 #ifndef WINDOWED_SUPPORT
@@ -356,10 +349,6 @@ void VideoDialog::loadConfig()
   myGLStretchCheckbox->setState(instance().settings().getBool("gl_fsmax"));
   myGLStretchCheckbox->setEnabled(gl);
 
-  // Use VBO (GL mode only)
-  myUseVBOCheckbox->setState(instance().settings().getBool("gl_vbo"));
-  myUseVBOCheckbox->setEnabled(gl);
-
   // Use sync to vertical blank (GL mode only)
   myUseVSyncCheckbox->setState(instance().settings().getBool("gl_vsync"));
   myUseVSyncCheckbox->setEnabled(gl);
@@ -416,13 +405,12 @@ void VideoDialog::saveConfig()
 
   // PAL color-loss effect
   instance().settings().setBool("colorloss", myColorLossCheckbox->getState());
+  if(&instance().console())
+    instance().console().toggleColorLoss(myColorLossCheckbox->getState());
 
   // GL stretch setting
   instance().settings().setBool("gl_fsmax", myGLStretchCheckbox->getState());
 
-  // Use VBO (GL mode only)
-  instance().settings().setBool("gl_vbo", myUseVBOCheckbox->getState());
-
   // Use sync to vertical blank (GL mode only)
   instance().settings().setBool("gl_vsync", myUseVSyncCheckbox->getState());
 
@@ -459,7 +447,6 @@ void VideoDialog::setDefaults()
   myFullscreenPopup->setSelected("0", "");
   myColorLossCheckbox->setState(true);
   myGLStretchCheckbox->setState(true);
-  myUseVBOCheckbox->setState(true);
   myUseVSyncCheckbox->setState(true);
   myUIMessagesCheckbox->setState(true);
   myCenterCheckbox->setState(false);
